/*
Simple proc that will return the next thing a player clicks on, blocking until then
Overrides:
	client/Click()
	client/Del()
Returns:
	on success: object clicked on
	on failure: null
*/

client
	proc
		GetClick(var/priority = 1, var/timeout = 36000, var/override = 0)
			var/GetClick/G = new(src, priority, override)
			if(G)
				return G.get(timeout)

// Forward the request to client/GetClick()
mob
	proc
		GetClick(var/priority = 1, var/timeout = 36000, var/override = 0)
			if(!client)
				return
			else
				return client.GetClick(priority, timeout)

//handler datum
GetClick
	var/priority = 1
	var/GetClickHelper/helper
	var/returnvalue
	// Whether this should prevent the object's normal Click() procedure from being called
	var/override = 0
	New(var/client/C, var/priority = 1, var/override = 0)
		..()
		helper = new()
		src.priority = priority
		src.override = override
		//if there's no waiting click, just enter us into the queue
		if(!C.clickwaiting)
			C.clickwaiting = src
		//if there is, and are a higher or equal priority, send it a null click and replace it
		else if(C.clickwaiting.priority <= priority)
			C.clickwaiting.receive(null)
			C.clickwaiting = src
		//if there is a waiting click and we are a lower priority, just delete us
		else
			del(src)
	proc
		get(var/timeout = 36000)
			//have the helper sleep until time is up or it is deleted
			helper.help(timeout)
			. = returnvalue
			del(src)
		receive(var/object)
			//interrupt the helper's help() proc by deleting it, allowing get to finish
			returnvalue = object
			del(helper)


//helper datum
GetClickHelper
	proc
		//his idea of helping is to sit around doing nothing
		help(var/timeout = 36000)
			sleep(timeout)


client
	var/GetClick/clickwaiting
	Click(object)
		if(clickwaiting)
			clickwaiting.receive(object)
			var/override = clickwaiting.override
			clickwaiting = null
			// If we had a click waiting and it said to override the normal Click() call, then don't call ..()
			if(!override)
				..()
		else
			..()
	Del()
		if(clickwaiting)
			del(clickwaiting)
		..()